# -*- coding: utf-8 -*-
#
# Copyright (c) 2006 by Blue Dynamics Alliance Klein und Partner KEG Austria
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

__author__ = """Robert Niederreiter <robertn@bluedynamics.net>"""
__docformat__ = 'plaintext'

import copy
import time
import calendar

from zope.interface import implements 
from Products.Five import BrowserView

from interfaces import IStatsView
from constants import *

class StatsView(BrowserView):
    """Logic for statistics template."""
    
    implements(IStatsView)
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.stats = False
        self.my = self._allocateMy()
        self.domain = False
    
    def initialize(self):
        if self.request.get('reload', None):
            self.context.reload(self._readDomain())
        
    def statsAvailable(self):
        return self.context.statsAvailable()
    
    def statsAllowed(self):
        if not self.getAllowedDomains():
            return False
        return True
    
    def domainSelected(self, domain):
        if domain == self._readDomain():
            return True
        return False
    
    def monthSelected(self, month):
        m = self.my[:2]
        if m == month:
            return True
        return False
    
    def yearSelected(self, year):
        y = self.my[2:]
        if y == year:
            return True
        return False
    
    def getGeneralLastModified(self):
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return 'NA'
        date = stats['GENERAL']['LastUpdate'][0]
        return self._parseDate(date)
    
    def getAllowedDomains(self):
        return self.context.getAllowedDomains()
    
    def getMonthSelection(self):
        return MONTH_TUPLES
    
    def getYearSelection(self):
        # do some logic here :))
        return ['2006', '2007']
            
    def getSiteUrl(self):
        return self._readDomain()
    
    def getEpoch(self):
        month = self.my[:2]
        year = self.my[2:]
        return '%s: %s %s' % ('Monat', MONTH[month], year)
    
    def getFirstTime(self):
        """Return the first access time or NA if there are no stats for
        requested month.
        """
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return 'NA'
        date = stats['GENERAL']['FirstTime']
        if len(date) == 0:
            return 'NA'
        return self._parseDate(date[0])
        
    def getLastTime(self):
        """Return the last access time or NA if there are no stats for
        requested month.
        """
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return 'NA'
        date = self.stats[self.my]['GENERAL']['LastTime']
        if len(date) == 0:
            return 'NA'
        return self._parseDate(date[0])
    
    def getTotalUnique(self, my=None):
        """Return total unique visits for requested month.
        """
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if stats:
            return stats['GENERAL']['TotalUnique'][0]
        return '0'
    
    def getTotalVisits(self, my=None):
        """Return total visits for requested month.
        """
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if stats:
            return stats['GENERAL']['TotalVisits'][0]
        return '0'
    
    def getTotalAverage(self):
        """Return average of visits by unique visitors for requested month.
        """
        self._readStats()
        unique = float(self.getTotalUnique())
        visits = float(self.getTotalVisits())
        if unique == 0:
            return '0'
        average = visits / unique
        return '%1.2f' % average

    def getTotalKnownPages(self, my=None):
        """Return total known pages requested for current month.
        """
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if not stats:
            return '0'
        days = stats['DAY']
        pages = 0
        for key in days.keys():
            pages += int(days[key]['pages'])
        return str(pages)
    
    def getTotalPagesAverage(self):
        """Return average of pages by visits for requested month.
        """
        self._readStats()
        visits = float(self.getTotalVisits())
        pages = float(self.getTotalKnownPages())
        if visits == 0:
            return '0'
        average = pages / visits
        return '%1.2f' % average
    
    def getTotalKnownHits(self, my=None):
        """Return total known hits for requested month.
        """
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if not stats:
            return '0'
        days = stats['DAY']
        hits = 0
        for key in days.keys():
            hits += int(days[key]['hits'])
        return str(hits)
    
    def getTotalHitsAverage(self):
        """Return average of hits by visits for requested month.
        """
        self._readStats()
        visits = float(self.getTotalVisits())
        hits = float(self.getTotalKnownHits())
        if visits == 0:
            return '0'
        average = hits / visits
        return '%1.2f' % average
    
    def getTotalKnownBytes(self, my=None):
        """Return total known bytes for requested month.
        """
        return self._parseBytes(self._getTotalRawBytes(my))
    
    def getTotalBytesAverage(self):
        """Return average of bytes by visits for requested month.
        """
        self._readStats()
        bytes = self._getTotalRawBytes()
        visits = int(self.getTotalVisits())
        if visits == 0:
            return '0'
        average = bytes / visits
        return self._parseBytes(average)
    
    def getTotalUnknownPages(self):
        """Return total count of unknown pages for requested month.
        """
        return ''
        # later
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return '0'
        visitor = self.stats[self.my]['CLUSTER']
        pages = 0
        for key in visitor.keys():
            pages += int(visitor[key]['pages'])
        return str(pages)
    
    def getTotalUnknownHits(self):
        """Return total unknown hits for requested month.
        """
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return '0'
        sections = ['ROBOT', 'WORMS', 'ERRORS']
        data = [self.stats[self.my][section] for section in sections]
        hits = 0
        for d in data:
            if not d:
                continue
            for key in d.keys():
                hits += int(d[key]['hits'])
        return str(hits)
    
    def getTotalUnknownBytes(self):
        """Return total unknown bytes for requested month.
        """
        self._readStats()
        stats = self.stats[self.my]
        if not stats:
            return '0'
        sections = ['ROBOT', 'WORMS', 'ERRORS']
        data = [self.stats[self.my][section] for section in sections]
        bytes = 0
        for d in data:
            if not d:
                continue
            for key in d.keys():
                bytes += int(d[key]['bandwidth'])
        return self._parseBytes(bytes)
    
    def getMonthGraph(self):
        data = self._getRawMonthData(self.my[2:])
        graph = dict()
        for set in data:
            graph[set['month']] = set['data']
            
        self._calculateGraphsData(graph, 100)
        for set in data:
            set['data'] = graph[set['month']]
        return data
    
    def getMonthBarNames(self):
        return ['unique', 'visit', 'page', 'hit', 'byte']
    
    def getMonthOverview(self):
        data = self._getRawMonthData(self.my[2:])
        for set in data:
            set['data']['byte'] = self._parseBytes(set['data']['byte'])
        return data
    
    def getMonthSum(self):
        data = self._getRawMonthData(self.my[2:])
        fields = self.getMonthBarNames()
        summary = dict()
        for field in fields:
            summary[field] = 0
        for set in data:
            for field in fields:
                summary[field] += set['data'][field]
        summary['byte'] = self._parseBytes(summary['byte'])
        return summary
    
    def getDaysInMonthGraph(self):
        data = self._getRawDayInMonthData()
        graph = dict()
        for set in data:
            graph[set['date']] = set['data']
        
        self._calculateGraphsData(graph, 60)
        for set in data:
            set['data'] = graph[set['date']]
        return data
    
    def getDaysInMonthBarNames(self):
        return ['visit', 'page', 'hit', 'byte']
    
    def getDayInMonthOverview(self):
        data = self._getRawDayInMonthData()
        for set in data:
            set['data']['byte'] = self._parseBytes(set['data']['byte'])
        return data
    
    def getDayInMonthAverage(self):
        data = self._getRawDayInMonthSum()
        days = calendar.mdays[11] # do some logic here
        for field in data.keys():
            data[field] = '%1.2f' % (float(data[field]) / days)
        data['byte'] = self._parseBytes(data['byte'])
        return data
    
    def getDayInMonthSum(self):
        data = self._getRawDayInMonthSum()
        data['byte'] = self._parseBytes(data['byte'])
        return data
    
    def getWeekdaysGraph(self):
        data = self._getRawWeekdaysData()
        graph = dict()
        for set in data:
            graph[set['day']] = set['data']
        
        self._calculateGraphsData(graph, 60)
        for set in data:
            set['data'] = graph[set['day']]
        return data
    
    def getWeekdaysBarNames(self):
        return ['page', 'hit', 'byte']
        
    def getWeekdaysOverview(self):
        data = self._getRawWeekdaysData()
        for set in data:
            set['data']['byte'] = self._parseBytes(set['data']['byte'])
        return data
    
    def getServertimeGraph(self):
        data = self._getRawServertimeData()
        graph = dict()
        for set in data:
            graph[set['hour']] = set['data']
        
        self._calculateGraphsData(graph, 60)
        for set in data:
            set['data'] = graph[set['hour']]
        return data
    
    def getServertimeBarNames(self):
        return ['page', 'hit', 'byte']
    
    def getServertimeOverview(self):
        data = self._getRawServertimeData()
        for set in data:
            set['data']['byte'] = self._parseBytes(set['data']['byte'])
        return data
    
    def getCountryData(self):
        data = self._getOrderedDomainData()
        graph = dict()
        for set in data:
            graph[set['domain']] = copy.deepcopy(set['data'])
        
        self._calculateGraphsData(graph, 150)
        for set in data:
            set['flag'] = 'flags/flag-%s.gif' % set['domain']
            set['country'] = DOMAINS.get(set['domain'].upper(), 'Undefined')
            set['text'] = set['data']
            set['text']['byte'] = self._parseBytes(set['data']['byte'])
            set['data'] = graph[set['domain']]
        
        #do some request logic here
        if len(data) > 10:
            return data[:10]
        
        return data
    
    def getCountryBarNames(self):
        return ['page', 'hit', 'byte']
    
    def getClientSummaryText(self):
        params = {
            'known': self._getKnownClientsCount(),
            'unknown' : self._getUnknownClientsCount(),
        }
        text = """Rechner: %(known)s Bekannte, %(unknown)s Unbekannte 
                  (IP konnte nicht aufgelöst werden )
                """ % params
        return text
    
    def getClientSummary(self):
        data = self._getOrderedClientData()
        for set in data:
            set['lastaccess'] = self._parseDate(set['lastaccess'])
            set['byte'] = self._parseBytes(set['byte'])
        
        #do some request logic here
        if len(data) > 10:
            return data[:10]
        
        return data
    
    def getRobotSummary(self):
        data = self._getOrderedRobotData()
        for set in data:
            set['lastaccess'] = self._parseDate(set['lastaccess'])
            set['byte'] = self._parseBytes(set['byte'])
        
        #do some request logic here
        if len(data) > 10:
            return data[:10]
        
        return data
    
    def getSessionSummary(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            rawdata = dict()
        else:
            rawdata = stats['SESSION']
            
        ranges = [ '0s-30s', '30s-2mn', '2mn-5mn', '5mn-15mn', 
                   '15mn-30mn', '30mn-1h', '1h+' ]
        
        total = int(self.getTotalVisits())
        known = 0
        for range in ranges:
            rangedata = rawdata.get(range, {})
            known += int(rangedata.get('number of visits', 0))
        unknown = total - known
            
        data = []
        for range in ranges:
            set = dict()
            rangedata = rawdata.get(range, {})
            visits = int(rangedata.get('number of visits', 0))
            percent = self._calculateProportion(total, visits) * 100
            set['name'] = range
            set['count'] = visits
            set['percent'] = '%1.2f %s' % (percent, '%')
            data.append(set)
        set = dict()
        percent = self._calculateProportion(total, unknown) * 100
        set['name'] = 'Unbekannt'
        set['count'] = unknown
        set['percent'] = '%1.2f %s' % (percent, '%')
        data.append(set)
        return data
    
    def getDatatypeSummary(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            rawdata = dict()
        else:
            rawdata = stats['FILETYPES']
        types = [ 'html', 'css', 'js', 'jpg', 'png',
                  'gif', 'pdf', 'doc', 'Unknown' ]
        totalhit = 0
        totalbyte = 0
        for type in types:
            typedata = rawdata.get(type, {})
            totalhit += int(typedata.get('hits', 0))
            totalbyte += int(typedata.get('bandwidth', 0))
        
        data = []
        for type in types:
            set = dict()
            typedata = rawdata.get(type, {})
            hit = int(typedata.get('hits', 0))
            hitpercent = self._calculateProportion(totalhit, hit) * 100
            byte = int(typedata.get('bandwidth', 0))
            bytepercent = self._calculateProportion(totalbyte, byte) * 100
            set['icon'] = '%s-ico.png' % type
            set['postfix'] = type
            set['filetype'] = FILETYPES[type]
            set['hit'] = hit
            set['hitpercent'] = '%1.2f %s' % (hitpercent, '%')
            set['byte'] = self._parseBytes(byte)
            set['bytepercent'] = '%1.2f %s' % (bytepercent, '%')
            data.append(set)
        return data
    
    def getSiteUrlInfoText(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        count = 0
        if stats:
            rawdata = stats['SIDER']
            count = len(rawdata.keys())
        text = '%i Unterschiedliche Seiten' % count
        return text
    
    def getSiteUrlData(self):
        data = self._getOrderedSiteUrlData()
        graph = dict()
        for set in data:
            name = set['name']
            graph[name] = dict()
            graph[name]['hit'] = set['hit']
            graph[name]['sizeaverage'] = set['sizeaverage']
            graph[name]['entrance'] = set['entrance']
            graph[name]['exit'] = set['exit']
        
        self._calculateGraphsData(graph, 150)
        for set in data:
            set['sizeaverage'] = self._parseBytes(set['sizeaverage'])
            set['data'] = graph[set['name']]
        
        #do some request logic here
        if len(data) > 10:
            return data[:10]
        
        return data
    
    def getSiteUrlBarNames(self):
        return [ 'hit', 'sizeaverage', 'entrance', 'exit' ]
    
    def getOsSummary(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            rawdata = dict()
        else:
            rawdata = stats['OS']
        total = { 'win': 0, 'mac': 0, 'linux': 0, 'Unknown': 0 }
        for os in rawdata.keys():
            for prefix in total.keys():
                if os.startswith(prefix):
                    total[prefix] += int(rawdata[os].get('hits', 0))
                    break
        
        totalsum = 0
        for key in total.keys():
            totalsum += total[key]

        data = []
        oskeys = ['win', 'mac', 'linux', 'Unknown']
        for os in oskeys:
            set = dict()
            set['icon'] = '%s-ico.png' % os
            set['name'] = OS_FAMILIES[os]
            hit = total[os] 
            set['hit'] = hit
            hitpercent = self._calculateProportion(totalsum, hit) * 100
            set['hitpercent'] = '%1.2f %s' % (hitpercent, '%')
            data.append(set)
        return data
    
    def getBrowserSummary(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            rawdata = dict()
        else:
            rawdata = stats['BROWSER']
        
        browserfamilies = BROWSER_FAMILIES.keys()
        
        total = dict()
        for browser in browserfamilies:
            total[browser] = 0
        
        for browser in rawdata.keys():
            for prefix in total.keys():
                if browser.startswith(prefix):
                    total[prefix] += int(rawdata[browser].get('hits', 0))
                    break
        
        totalsum = 0
        for key in total.keys():
            totalsum += total[key]

        data = []
        for browser in browserfamilies:
            set = dict()
            set['icon'] = '%s-ico.png' % browser
            set['name'] = BROWSER_FAMILIES[browser]
            hit = total[browser] 
            set['hit'] = hit
            hitpercent = self._calculateProportion(totalsum, hit) * 100
            set['hitpercent'] = '%1.2f %s' % (hitpercent, '%')
            data.append(set)
        return data

    def _allocateMy(self):
        """Allocate month/year for this request.
        
        Try to get month/year from request, if successless return curren month
        and year.
        """
        req = self.request
        month = req.get('currentmonth', None)
        year = req.get('currentyear', None)
        if not month or not year:
            lt = time.localtime()
            month = str(lt[1])
            year = str(lt[0])
            if len(month) == 1:
                month = '0%s' % month
        return '%s%s' % (month, year)
        
    def _currentMy(self, my):
        """Check if month / year is not the default for this request.
        
        Needed for the month overview statistics f.e."""
        if my is None:
            return self.my
        return my
        
    def _readStats(self):
        """Read the statistics.
        """
        if self.stats is False:
            self.stats = self.context.getStatistics(self._readDomain())
    
    def _readDomain(self):
        if self.domain is False:
            allowed = self.getAllowedDomains()
            domain = self.request.get('domain')
            if not domain or not domain in allowed:
                self.domain = allowed[0]
            else:
                self.domain = domain
        return self.domain
            
    def _getTotalRawBytes(self, my=None):
        """Return the total raw bytes for this month.
        """
        self._readStats()
        bytes = 0
        my = self._currentMy(my)
        stats = self.stats[my]
        
        if not stats:
            return bytes
        
        days = stats['DAY']
        for key in days.keys():
            bytes += int(days[key]['bandwidth'])
        return bytes
    
    def _getRawMonthData(self, year):
        """Return the raw (not formatted) general data for this month.
        """
        self._readStats()
        data = []
        month = MONTH.keys()
        month.sort()
        
        for m in month:
            my = '%s%s' % (m, year)
            set = dict()
            set['month'] = '%s %s' % (MONTH[m], year)
            set['data'] = dict()
            set['data']['unique'] = int(self.getTotalUnique(my))
            set['data']['visit'] = int(self.getTotalVisits(my))
            set['data']['page'] = int(self.getTotalKnownPages(my))
            set['data']['hit'] = int(self.getTotalKnownHits(my))
            set['data']['byte'] = self._getTotalRawBytes(my)
            data.append(set)
        return data
    
    def _getRawDayInMonthData(self):
        my = self.my
        mdays = self._getMdaysFor(my)
        data = []
        dateday = 1
        for day in mdays:
            set = dict()
            set['date'] = self._getDateForDay(my, dateday)
            set['dateday'] = str(dateday)
            set['weekday'] = DAYS[day]
            set['data'] = dict()
            set['data']['visit'] = self._getFieldForDay(my, dateday, 'visits')
            set['data']['page'] = self._getFieldForDay(my, dateday, 'pages')
            set['data']['hit'] = self._getFieldForDay(my, dateday, 'hits')
            set['data']['byte'] = self._getFieldForDay(my, dateday, 'bandwidth')
            if day == 6 or day == 0:
                set['highlight'] = True
            else:
                set['highlight'] = False
            data.append(set)
            dateday += 1
        return data
    
    def _getRawDayInMonthSum(self):
        data = self._getRawDayInMonthData()
        fields = self.getDaysInMonthBarNames()
        summary = dict()
        for field in fields:
            summary[field] = 0
        for set in data:
            for field in fields:
                summary[field] += set['data'][field]
        return summary
    
    def _getMdaysFor(self, my):
        month = int(my[:2])
        year = int(my[2:])
        wdays = calendar.monthcalendar(year, month)
        mdays = []
        for week in wdays:
            wday = 0
            for day in week:
                if day > 0:
                    mdays.append(wday)
                wday += 1
        return mdays
    
    def _getDateForDay(self, my, day):
        date = self._getRawDate(my, day)
        return self._parseDate(date, longformat=False)
    
    def _getFieldForDay(self, my, day, field):
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if not stats:
            return 0
        data = stats['DAY'].get(self._getRawDate(my, day))
        if data:
            field = data.get(field)
            if field:
                return int(field)
        return 0
    
    def _getRawWeekdaysData(self):
        rawdata = self._getRawDayInMonthData()        
        weekdays = ['Mo', 'Di', 'Mi', 'Do', 'Fr', 'Sa', 'So']
        setdataFields = self.getWeekdaysBarNames()
        weekdaysums = dict()
        for weekday in weekdays:
            weekdaysums[weekday] = dict()
            for field in setdataFields:
                weekdaysums[weekday][field] = 0
            weekdaysums[weekday]['count'] = 0.0
        
        for set in rawdata:
            setdata = set['data']
            for field in setdataFields:
                weekdaysums[set['weekday']][field] += setdata[field]
            weekdaysums[set['weekday']]['count'] += 1
        
        data = []
        for day in weekdays:
            set = dict()
            set['day'] = day
            set['data'] = dict()
            for field in setdataFields:
                value = weekdaysums[day][field] / weekdaysums[day]['count']
                set['data'][field] = value
            if day == 'Sa' or day == 'So':
                set['highlight'] = True
            else:
                set['highlight'] = False
            data.append(set)   
        return data
    
    def _getRawServertimeData(self):
        my = self.my
        hours = HOURS
        setdataFields = ['page', 'hit', 'byte']
        statFields = ['pages', 'hits', 'bandwidth']
        
        hoursums = dict()
        for hour in hours:
            hoursums[hour] = dict()
            for field in statFields:
                value = self._getFieldForServertime(my, hour, field)
                hoursums[hour][field] = value
        
        data = []
        for hour in hours:
            set = dict()
            set['hour'] = hour
            set['data'] = dict()
            p = 0
            for field in setdataFields:
                value = hoursums[hour][statFields[p]]
                set['data'][field] = value
                p += 1
            data.append(set)
        return data
    
    def _getFieldForServertime(self, my, hour, field):
        self._readStats()
        my = self._currentMy(my)
        stats = self.stats[my]
        if not stats:
            return 0
        data = stats['TIME'].get(hour)
        if data:
            field = data.get(field)
            if field:
                return int(field)
        return 0
    
    def _getOrderedDomainData(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            return []
        rawdata = stats['DOMAIN']
        domains = rawdata.keys()
        sortlist = []
        for domain in domains:
            sortlist.append(int(rawdata[domain]['pages']))
        sortlist.sort()
        sortlist.reverse()
        data = []
        for entry in sortlist:
            entry = str(entry)
            p = 0
            for domain in domains:
                if rawdata[domain]['pages'] == entry:
                    set = dict()
                    set['domain'] = domain
                    set['data'] = dict()
                    set['data']['page'] = int(rawdata[domain]['pages'])
                    set['data']['hit'] = int(rawdata[domain]['hits'])
                    set['data']['byte'] = int(rawdata[domain]['bandwidth'])
                    domains.pop(p)
                    data.append(set)
                    break
                p += 1
        return data
    
    def _getOrderedClientData(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            return []
        rawdata = stats['VISITOR']
        clients = rawdata.keys()
        
        sortlist = []
        for client in clients:
            sortlist.append(int(rawdata[client]['pages']))
        sortlist.sort()
        sortlist.reverse()
        data = []
        
        for entry in sortlist:
            entry = str(entry)
            p = 0
            for client in clients:
                if rawdata[client]['pages'] == entry:
                    clientdata = rawdata[client]
                    set = dict()
                    set['name'] = client
                    set['page'] = int(clientdata.get('pages', 0))
                    set['hit'] = int(clientdata.get('hits', 0))
                    set['byte'] = int(clientdata.get('bandwidth', 0))
                    set['lastaccess'] = clientdata.get('last visit date', '')
                    clients.pop(p)
                    data.append(set)
                    break
                p += 1
        return data
    
    def _getKnownClientsCount(self):
        data = self._getOrderedClientData()
        count = 0
        for set in data:
            client = set['name']
            if self._isClientKnown(client):
                count += 1
        return count
    
    def _isClientKnown(self, client):
        splitted = client.split('.')
        for part in splitted:
            if not part.isdigit():
                return True
        return False
    
    def _getUnknownClientsCount(self):
        data = self._getOrderedClientData()
        known = self._getKnownClientsCount()
        return len(data) - known
    
    def _getOrderedRobotData(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            return []
        rawdata = stats['ROBOT']
        robots = rawdata.keys()
        
        sortlist = []
        for robot in robots:
            sortlist.append(int(rawdata[robot]['hits']))
        sortlist.sort()
        sortlist.reverse()
        data = []
        
        for entry in sortlist:
            entry = str(entry)
            p = 0
            for robot in robots:
                if rawdata[robot]['hits'] == entry:
                    robotdata = rawdata[robot]
                    set = dict()
                    set['name'] = robot
                    set['hit'] = int(robotdata.get('hits', 0))
                    set['byte'] = int(robotdata.get('bandwidth', 0))
                    set['lastaccess'] = robotdata.get('last visit', '')
                    robots.pop(p)
                    data.append(set)
                    break
                p += 1
        return data
    
    def _getOrderedSiteUrlData(self):
        self._readStats()
        my = self.my
        stats = self.stats[my]
        if not stats:
            return []
        rawdata = stats['SIDER']
        siteurls = rawdata.keys()
        sortlist = []
        for siteurl in siteurls:
            pages = rawdata[siteurl].get('pages', 0)
            sortlist.append(int(rawdata[siteurl]['pages']))
        sortlist.sort()
        sortlist.reverse()
        data = []
        for entry in sortlist:
            entry = str(entry)
            p = 0
            for siteurl in siteurls:
                if rawdata[siteurl]['pages'] == entry:
                    set = dict()
                    set['name'] = siteurl
                    hit = int(rawdata[siteurl].get('pages', 0))
                    set['hit'] = hit
                    byte = int(rawdata[siteurl].get('bandwidth'), 0)
                    set['sizeaverage'] = byte / hit
                    set['entrance'] = int(rawdata[siteurl].get('entry', 0))
                    set['exit'] = int(rawdata[siteurl].get('exit', 0))
                    siteurls.pop(p)
                    data.append(set)
                    break
                p += 1
        return data
    
    def _calculateGraphsData(self, data, size):
        # data is a dict withs graphs
        # size is the max size for a bar in pixel
        # every graph is a dict itself with the bars
        # the graphs inside data must have the same structure
        if not data:
            return None
        
        graphs = data.keys()
        maxvalues = dict()
        for bar in data[graphs[0]].keys():
            maxvalues[bar] = 0
        
        for graph in graphs:
            for bar in data[graph].keys():
                value = data[graph][bar]
                if value > maxvalues[bar]:
                    maxvalues[bar] = value
        
        for graph in graphs:
            self._calculateGraphData(data[graph], size, maxvalues)
    
    def _calculateGraphData(self, graph, size, maxvalues):
        bars = graph.keys()
        for bar in bars:
            prop = self._calculateProportion(maxvalues[bar], graph[bar])
            graph[bar] = int(size * prop)
            
    def _calculateProportion(self, par, comp):
        par = float(par)
        comp = float(comp)
        if par == 0:
            return 0
        return comp / par
    
    def _getRawDate(self, my, day):
        my = str(my)
        day = str(day)
        year = my[2:]
        month = my[:2]
        if len(day) == 1:
            day = '0%s' % day
        return '%s%s%s' % (year, month, day)
    
    def _parseDate(self, date, longformat=True):
        if not date:
            return ''
        year = date[:4]
        month = date[4:6]
        day = date[6:8]
        if longformat:
            hour = date[8:10]
            minute = date[10:12]
            return '%s.%s.%s - %s:%s' % (day, month, year, hour, minute)
        return '%s.%s.%s' % (day, month, year)
    
    def _parseBytes(self, bytes):
        unit = 0
        volume = float(bytes)
        while True:
            if volume / 1024 < 1:
                return '%1.2f%s' % (volume, DATA_UNITS[unit])
            volume = volume / 1024
            unit += 1