# -*- coding: utf-8 -*-
#
# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#

"""
CMFBlueAwstatsManagement module

Defines the CMFBlueAwstatsManagementTool class

depends on CMF.
"""

__author__ = """Robert Niederreiter <robertn@bluedynamics.com>"""
__docformat__ = 'plaintext'

from Globals import DTMLFile, MessageDialog, Persistent
from OFS.SimpleItem import Item
from Acquisition import Implicit, aq_inner, aq_parent
from ZPublisher import BeforeTraverse
from zExceptions import BadRequest
from urllib import quote
from ZODB.POSException import ConflictError
from Products.CMFCore.utils import getToolByName
from persistent.dict import PersistentDict

from bda.awstatsparser.defaults import *
from bda.awstatsparser.parser import ParsedStatistics

import os, sys
import types

try:
    from zExceptions import NotFound
except ImportError:
    NotFound = 'NotFound'

class BlueAwstatsManagementTool(Persistent, Item, Implicit):
    """Tool for managing allowed statistics in a specific aquisition context
    """

    meta_type='BlueAwstats Management Tool'

    title = 'BlueAwstats Management Tool'
    
    allowed_domains = []
    allowed_domains_for_user = {}
    statistics = {}
    basedir = '/var/lib/awstats'
    prefix = 'awstats'
    postfix = 'txt'

    __ac_permissions__ = (
        ('View', ('manage_main',)),
        ('Add Site Roots', ('manage_basesettings', 'manage_domains'))
    )

    manage_options=(
        {
            'label':'BlueAwstats Management Tool',
            'action':'manage_main',
        },
        {
            'label':'Base Settings',
            'action':'manage_basesettings',
        },
        {
            'label':'Manage Domains',
            'action':'manage_domains',
        },
        {
            'label':'User Domains',
            'action':'manage_user_domains',
        },
    )

    manage_main = DTMLFile('www/BlueAwstatsManagementTool', globals(),
                           __name__='manage_main')
    manage_domains = DTMLFile('www/manage_domains', globals())
    manage_user_domains = DTMLFile('www/manage_user_domains', globals())
    manage_basesettings = DTMLFile('www/manage_basesettings', globals())
            
    def purge(self, domain=None):
        if not domain:
            self.statistics = {}
        else:
            try:
                del self.statistics[domain]
            except KeyError: 
                pass
    
    def getAllowedDomains(self):
        """Return the allowed Domains
        """
        pm = getToolByName(self, 'portal_membership', None)
        if pm is not None:
            current = pm.getAuthenticatedMember().getUserName()
            allowed_domains_for_user = self._getAllowedDomainsForUserDict()
            return allowed_domains_for_user.get(current, [])
        else:
            return self.allowed_domains
    
    def readStatistics(self, domain):
        """Read statistics for domain
        """
        if self.isBaseDirAvailable():
            dir = self.basedir
            pre = self.prefix
            post = self.postfix
            stats = ParsedStatistics(domain, dir, pre, post, SECTIONDEFS)
            self.statistics[domain] = stats
    
    def getStatistics(self, domain):
        """returns the statistics.
        """
        stats = self.statistics.get(domain)
        if not stats:
            self.readStatistics(domain)
        return self.statistics.get(domain)
    
    def isNotBaseDirAvailable(self):
        "Return True if basedir is not available"
        if os.path.exists(self.basedir):
            return False
        return True
    
    def isBaseDirAvailable(self):
        "Return True if basedir is available"
        return os.path.exists(self.basedir)
    
    def listAvailableDomains(self):
        if self.isNotBaseDirAvailable():
            return []
        dircontents = os.listdir(self.basedir)
        files = []
        for entry in dircontents:
            if not os.path.isdir(entry):
                files.append(entry)
        domains = []
        for file in files:
            if not file.startswith(self.prefix) \
              or not file.endswith(self.postfix):
                continue
            start = len(self.prefix) + 7
            end = file.index('.%s' % self.postfix)
            domain = file[start:end]
            if not domain in domains:
                domains.append(domain)
        return domains
    
    def setBaseProperties(self, basedir, prefix, postfix, RESPONSE=None):
        "Set the baseproperties for this awstats management tool"
        self.basedir = basedir
        self.prefix = prefix
        self.postfix = postfix
        
        if RESPONSE is not None:
            RESPONSE.redirect('manage_basesettings?manage_tabs_message=Changes%20Saved.')
    
    def setAllowedDomains(self, domains=[], RESPONSE=None):
        "Set the domains allowed to view the statistics of."
        if type(domains) is types.StringType:
            domains = [domains]
        self.allowed_domains = domains
        if RESPONSE is not None:
            RESPONSE.redirect('manage_domains?manage_tabs_message=Changes%20Saved.')
    
    def isUserManagementAvailable(self):
        """Lookup for a plone portal_membership tool
        """
        pm = getToolByName(self, 'portal_membership', None)
        if pm is None:
            return False
        return True
    
    def getAvailableUsers(self):
        """Return the available plone users.
        """
        pm = getToolByName(self, 'portal_membership', None)
        members = pm.listMembers()
        ids = []
        for member in members:
            ids.append(member.getId())
        return ids
    
    def getAllowedDomainsForUser(self, userid):
        allowed_domains_for_user = self._getAllowedDomainsForUserDict()
        return allowed_domains_for_user.get(userid, [])
    
    def setAllowedDomainsForUsers(self, userid=[], domains=[], RESPONSE=None):
        "Set the domains allowed to view the statistics of for specific users."
        allowed_domains_for_user = self._getAllowedDomainsForUserDict()
        if type(userid) is types.StringType:
            userid = [userid]
        if type(domains) is types.StringType:
            domains = [domains]
        for id in userid:
            allowed_domains_for_user[id] = domains
        if RESPONSE is not None:
            RESPONSE.redirect('manage_user_domains?manage_tabs_message=Changes%20Saved.')
    
    def _getAllowedDomainsForUserDict(self):
        if not self.allowed_domains_for_user:
            self.allowed_domains_for_user = PersistentDict()
        return self.allowed_domains_for_user

    def addToContainer(self, container):
        container._setObject(self.id, self)

    def manage_addToContainer(self, container, nextURL=''):
        self.addToContainer(container)
        if nextURL:
            return MessageDialog(title='Item Added',
              message='This object now has a %s' % self.meta_type,
              action=nextURL)

    def manage_beforeDelete(self, item, container):
        if item is self:
            BeforeTraverse.unregisterBeforeTraverse(container, self.meta_type)

def manage_addCMFBlueAwstatsManagementTool(self, REQUEST=None, **ignored):
    """ """
    bamt = BlueAwstatsManagementTool()
    bamt.id = str('blueawstats_management_tool')
    if REQUEST:
        return bamt.manage_addToContainer(self.this(),
                                        '%s/manage_main' % REQUEST['URL1'])
    else:
        bamt.addToContainer(self.this())

constructors = (
    (
        'manage_addBlueAwstatsManagementToolForm',
        DTMLFile('www/BlueAwstatsManagementToolAdd',
        globals())
    ),
    (
        'manage_addBlueAwstatsManagementTool',
        manage_addCMFBlueAwstatsManagementTool
    ),
)
 